import { ZERO } from '@/constants/common';
import type { GetHex3ColorGradientColorWithAlpha } from '@/hooks/useTriColorLerp';
import { OverlayOrder } from '@/redux/overlayBioEntity/overlayBioEntity.utils';
import { OverlayBioEntityRender } from '@/types/OLrendering';
import { LinePoint } from '@/types/reactions';
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import type Feature from 'ol/Feature';
import { Geometry } from 'ol/geom';
import type Polygon from 'ol/geom/Polygon';
import { createOverlayGeometryFeature } from './createOverlayGeometryFeature';
import { createOverlayLineFeature } from './createOverlayLineFeature';
import { getColorByAvailableProperties } from './getColorByAvailableProperties';
import { getPolygonLatitudeCoordinates } from './getPolygonLatitudeCoordinates';

type GetOverlayFeaturesProps = {
  bioEntities: OverlayBioEntityRender[];
  pointToProjection: UsePointToProjectionResult;
  getHex3ColorGradientColorWithAlpha: GetHex3ColorGradientColorWithAlpha;
  defaultColor: string;
  overlaysOrder: OverlayOrder[];
};

export const getOverlayFeatures = ({
  bioEntities,
  pointToProjection,
  getHex3ColorGradientColorWithAlpha,
  defaultColor,
  overlaysOrder,
}: GetOverlayFeaturesProps): Feature<Polygon>[] | Feature<Geometry>[] =>
  bioEntities.map(entity => {
    /**
     * Depending on number of active overlays
     * it's required to calculate xMin and xMax coordinates of the polygon
     * so "entity" might be devided equali between active overlays
     */
    const { xMin, xMax } = getPolygonLatitudeCoordinates({
      width: entity.width,
      nOverlays: overlaysOrder.length,
      xMin: entity.x1,
      overlayIndexBasedOnOrder:
        overlaysOrder.find(({ id }) => id === entity.overlayId)?.index || ZERO,
    });

    const color = getColorByAvailableProperties(
      entity,
      getHex3ColorGradientColorWithAlpha,
      defaultColor,
    );

    if (entity.type === 'rectangle') {
      return createOverlayGeometryFeature(
        [
          ...pointToProjection({ x: xMin, y: entity.y1 }),
          ...pointToProjection({ x: xMax, y: entity.y2 }),
        ],
        color,
      );
    }

    return createOverlayLineFeature(
      [
        { x: xMin, y: entity.y1 },
        { x: xMax, y: entity.y2 },
      ] as LinePoint,
      {
        color,
        pointToProjection,
      },
    );
  });
