import { useTriColorLerp } from '@/hooks/useTriColorLerp';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import {
  getOverlayOrderSelector,
  overlayBioEntitiesForCurrentModelSelector,
} from '@/redux/overlayBioEntity/overlayBioEntity.selector';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import { Feature } from 'ol';
import { Geometry } from 'ol/geom';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import { useMemo } from 'react';
import { getOverlayFeatures } from './getOverlayFeatures';

/**
 * Prerequisites: "view" button triggers opening overlays -> it triggers downloading overlayBioEntityData for given overlay for ALL available submaps(models)
 *
 * 1. For each active overlay
 * 2. get overlayBioEntity data (current map data passed by selector)
 * 3. based on nOverlays, calculate coordinates for given overlayBioEntity to render Polygon from extend
 * 4. Calculate coordinates in following steps:
 *   - polygonWidth = width/nOverlays
 *   - xMin = xMin + polygonWidth * overlayIndexBasedOnOrder
 *   - xMax = xMin + polygonWidth
 *   - yMin,yMax -> is const taken from store
 * 5. generate Feature(xMin,yMin,xMax,yMax)
 */

export const useOlMapOverlaysLayer = (): VectorLayer<VectorSource<Feature<Geometry>>> => {
  const pointToProjection = usePointToProjection();
  const { getHex3ColorGradientColorWithAlpha, defaultColorHex } = useTriColorLerp();
  const bioEntities = useAppSelector(overlayBioEntitiesForCurrentModelSelector);
  const overlaysOrder = useAppSelector(getOverlayOrderSelector);

  const features = useMemo(
    () =>
      getOverlayFeatures({
        bioEntities,
        pointToProjection,
        getHex3ColorGradientColorWithAlpha,
        defaultColor: defaultColorHex,
        overlaysOrder,
      }),
    [
      bioEntities,
      getHex3ColorGradientColorWithAlpha,
      pointToProjection,
      defaultColorHex,
      overlaysOrder,
    ],
  );

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features,
    });
  }, [features]);

  const overlaysLayer = useMemo(
    () =>
      new VectorLayer({
        source: vectorSource,
      }),
    [vectorSource],
  );

  return overlaysLayer;
};
